local _print = print
local Console = require "console"
console = nil
input   = nil
output  = nil

local console_active = true

local debug = debug
local function error_handler(msg)
	print((debug.traceback("Error: " .. msg, 4):gsub("\t", "    ")))
end

function love.draw()

end

function love.update()

end

function love.run()
	love.keyboard.setKeyRepeat(0.15, 0.025)
	local font = love.graphics.newFont('VeraMono.ttf', 14)
	console = Console.new(font)
	print("|  '/\\'\\  / /_\\  __  /' /\\ |\\ | /_ /\\ |   /_\\")
	print("|__ \\/  \\/  \\_       \\, \\/ | \\|  / \\/ |__ \\_   (v.1e-126)")
	print()
	print("<Escape> toggles the console. Call quit() or exit() to quit.")
	print("Try hitting <Tab> to complete your current input.")
	print("A leading '=' prints the calling result")
	print("You can overwrite every love callback (but love.keypressed).")
	print()

	math.randomseed(os.time())
	math.random() math.random()

	if love.load then love.load(arg) end

	local dt = 0

	-- Main loop time.
	while true do
		-- Process events.
		if love.event then
			love.event.pump()
			for e,a,b,c,d in love.event.poll() do
				if e == "quit" then
					if not love.quit or not love.quit() then
						if love.audio then
							love.audio.stop()
						end
						return
					end
				elseif e == "keypressed" then
					-- keypress event. let us catch that
					if a == 'escape' then
						console_active = not console_active
					end

					if console_active then
						console:keypressed(a,b)
					else
						love.handlers[e](a,b,c,d)
					end
					--end
				else
					-- call handlers in protective mode. if they fail,
					-- show error and remove the handler
					local f = love.handlers[e]
					if type(f) == "function" and not xpcall(function() f(a,b,c,d) end, error_handler) then
						love.handlers[e] = nil
					end
				end
			end
		end

		-- Update dt, as we'll be passing it to update
		if love.timer then
			love.timer.step()
			dt = love.timer.getDelta()
		end

		-- Call update and draw
		if love.update then love.update(dt) end -- will pass 0 if love.timer is disabled
		if love.graphics then
			love.graphics.clear()
			if love.draw then
				love.draw()
				-- draw console
				if console_active then
					local color = {love.graphics.getColor()}
					love.graphics.setColor(34,34,34,180)
					love.graphics.rectangle('fill', 2,2, love.graphics.getWidth()-4, love.graphics.getHeight()-4)
					love.graphics.setColor(240,240,0)
					console:draw(4, love.graphics.getHeight() - 4)
					love.graphics.setColor(unpack(color))
				end
			end
		end

		if love.timer then
			if console_active then
			-- to reduce CPU load we limit to 60 FPS
				love.timer.sleep(0.01666)
			else
				love.timer.sleep(0.001)
			end
		end
		if love.graphics then love.graphics.present() end

	end
end

function print(...)
	return console:print(...)
end

function printf(fmt, ...)
	return print(string.format(fmt, ...))
end

function quit()
	love.event.push('quit')
end
exit = quit
